@@ -27,11 +27,7 @@
  * 
  */
 #include <stdio.h>
-#ifdef __STDC__
 #include <errno.h>
-#else /* __STDC__ */
-#include <sys/errno.h>
-#endif /* __STDC__ */
 
 void describe_error(void)
 {
