@@ -14,14 +14,7 @@
 #include <sys/ioctl.h>
 #include <sys/soundcard.h>
 #include "fmtheaders.h"
-
-/* trying to define independent ioctl for sounddrivers version 1 and 2+ ,
-   but it dosn't work everywere */
-#ifdef SOUND_VERSION
-#define IOCTL(a,b,c)		ioctl(a,b,&c)
-#else
-#define IOCTL(a,b,c)		(c = ioctl(a,b,c) )
-#endif
+#include <netinet/in.h>
 
 #define DEFAULT_DSP_SPEED 	8000
 
@@ -35,6 +28,7 @@
 #define VOC_FMT			0
 #define WAVE_FMT		1
 #define RAW_DATA		2
+#define SND_FMT			3
 
 /* global data */
 
@@ -42,6 +36,7 @@ int timelimit = 0, dsp_speed = DEFAULT_DSP_SPEED, dsp_stereo = 0;
 int samplesize   = 8;
 int quiet_mode   = 0;
 int verbose_mode = 0;
+int convert      = 0;
 int record_type  = VOC_FMT;
 u_long count;
 int audio, abuf_size, zbuf_size;
@@ -60,8 +55,11 @@ struct {
 void record_play (char *name);
 void start_voc (int fd, u_long count);
 void start_wave (int fd, u_long count);
+void start_snd (int fd, u_long count);
 void end_voc (int fd);
 void end_wave_raw (int fd);
+void end_snd (int fd);
+u_long calc_count ();
 
 struct fmt_record {
   void (*start) (int fd, u_long count);
@@ -70,12 +68,13 @@ struct fmt_record {
 } fmt_rec_table[] = {
   { start_voc,  end_voc,      "VOC" },
   { start_wave, end_wave_raw, "WAVE" },
-  { NULL,       end_wave_raw, "raw data" }
+  { NULL,       end_wave_raw, "raw data" },
+  { start_snd,  end_snd,      "SND" }
 }; 
 
 int main (int argc, char *argv[])
 {
-  int c;
+  int c, n=0;
 
   command = argv[0];
   if (strstr (argv[0], "vrec")) {
@@ -92,7 +91,7 @@ int main (int argc, char *argv[])
     exit (1);
   }
 
-  while ((c = getopt (argc, argv, "qs:St:b:vrwd")) != EOF)
+  while ((c = getopt (argc, argv, "aqs:St:b:vrwd")) != EOF)
     switch (c) {
       case 'S':
 	dsp_stereo = raw_info.dsp_stereo = 1;
@@ -109,6 +108,9 @@ int main (int argc, char *argv[])
       case 'w' :
         record_type = WAVE_FMT;
         break;
+      case 'a' :
+        record_type = SND_FMT;
+        break;
       case 's':
 	dsp_speed = atoi (optarg);
 	if (dsp_speed < 300)
@@ -126,7 +128,7 @@ int main (int argc, char *argv[])
         break;
       default:
 	fprintf (stderr, 
-	 "Usage: %s [-qvwrS] [-t secs] [-s Hz] [-b 8|12|16] [filename]\n", command);
+	 "Usage: %s [-qvwraS] [-t secs] [-s Hz] [-b 8|12|16] [filename]\n", command);
 	exit (-1);
     }
 
@@ -136,13 +138,15 @@ int main (int argc, char *argv[])
     exit (-1);
   }
 
-  IOCTL(audio, SNDCTL_DSP_GETBLKSIZE, abuf_size);
+  abuf_size = -1;
+  ioctl(audio, SNDCTL_DSP_GETBLKSIZE, &abuf_size);
   if (abuf_size < 4 || abuf_size > 65536) {
     if (abuf_size == -1)
+    {
       perror (AUDIO);
-    else
-      fprintf (stderr, "Invalid audio buffers size %d\n", abuf_size);
-    exit (-1);
+      exit (-1);
+    }
+    abuf_size = 1024;
   }
  
   zbuf_size = 256;
@@ -157,6 +161,13 @@ int main (int argc, char *argv[])
     record_play (NULL);
   else
     while (optind <= argc - 1) {
+  
+      if (n++ > 0)
+      if (ioctl (audio, SNDCTL_DSP_SYNC, NULL) < 0) {
+         perror(AUDIO);
+         exit (-1);
+      }
+
       record_play (argv[optind++]);
     }
   close (audio);
@@ -170,7 +181,7 @@ int main (int argc, char *argv[])
 int test_vocfile(void *buffer)
 {
   VocHeader *vp = buffer;
-  if (strstr((const char *)vp->magic, MAGIC_STRING) ) {
+  if (!strcmp((const char *)vp->magic, MAGIC_STRING) ) {
     vocminor = vp->version & 0xFF;
     vocmajor = vp->version / 256;
     if (vp->version != (0x1233 - vp->coded_ver) )
@@ -206,6 +217,60 @@ int test_wavefile(void *buffer)
   }
   return -1;
 }
+
+/*
+* test, if it's a .SND file, 0 if ok (and set the speed, stereo etc.)
+*                          < 0 if not
+*/
+int test_sndfile(void *buffer, int fd)
+{
+  long infolen;
+  char *info;
+  SndHeader *snd = buffer;
+
+  if (snd->magic == SND_MAGIC)
+    convert = 0;
+  else{
+    if( htonl(snd->magic) == SND_MAGIC){
+      convert = 1;
+      snd->dataLocation = htonl(snd->dataLocation);
+      snd->dataSize = htonl(snd->dataSize);
+      snd->dataFormat = htonl(snd->dataFormat);
+      snd->samplingRate = htonl(snd->samplingRate);
+      snd->channelCount = htonl(snd->channelCount);
+    }
+    else{
+      /* No SoundFile */
+      return(-1);
+    }     
+  }     
+  switch (snd->dataFormat){
+  case SND_FORMAT_LINEAR_8:
+    samplesize = 8;
+    break;
+  case SND_FORMAT_LINEAR_16:
+    samplesize = 16;
+    break;
+  default:
+    fprintf (stderr, "%s: Unsupported SND_FORMAT\n", command);
+    exit (-1);
+  }
+  
+  dsp_stereo = (snd->channelCount == 2) ? 1 : 0;
+  dsp_speed = snd->samplingRate; 
+  count = snd->dataSize;
+  
+  /* read Info-Strings */
+  infolen = snd->dataLocation - sizeof(SndHeader);
+  info = (char *) malloc(infolen);
+  read(fd, info, infolen);
+  if(!quiet_mode)
+    fprintf(stderr, "SoundFile Info: %s\n", info);
+  free(info);
+
+  return 0;
+}
+
  
 /*
   writing zeros from the zerobuf to simulate silence,
@@ -231,16 +296,18 @@ void write_zeros (unsigned x)
 void 
 sync_dsp(void)
 {
+#if 0
   if (ioctl (audio, SNDCTL_DSP_SYNC, NULL) < 0) {
     perror(AUDIO);
     exit (-1);
   }
+#endif
 }
 
 /* setting the speed for output */
-void set_dsp_speed (int dsp_speed)
+void set_dsp_speed (int *dsp_speed)
 {
-  if (IOCTL(audio, SNDCTL_DSP_SPEED, dsp_speed) < 0) {
+  if (ioctl(audio, SNDCTL_DSP_SPEED, dsp_speed) < 0) {
     fprintf (stderr, "%s: unable to set audio speed\n", command);
     perror (AUDIO);
     exit (-1);
@@ -322,14 +389,14 @@ void vplay (int fd, int ofs, char *name)
 
   /* .voc files are 8 bit (now) */
   samplesize = VOC_SAMPLESIZE;
-  IOCTL(audio, SNDCTL_DSP_SAMPLESIZE, samplesize);
+  ioctl(audio, SNDCTL_DSP_SAMPLESIZE, &samplesize);
   if (samplesize != VOC_SAMPLESIZE) {
     fprintf(stderr, "%s: unable to set 8 bit sample size!\n", command);
     exit (-1);
   }
   /* and there are MONO by default */
   dsp_stereo = MODE_MONO;
-  IOCTL(audio, SNDCTL_DSP_STEREO, dsp_stereo);
+  ioctl(audio, SNDCTL_DSP_STEREO, &dsp_stereo);
 
   in_buffer = nextblock = 0;
   while (1) {
@@ -376,7 +443,7 @@ void vplay (int fd, int ofs, char *name)
             }
             if (dsp_stereo) {	/* if we are in Stereo-Mode, switch back */
               dsp_stereo = MODE_MONO;
-              IOCTL(audio, SNDCTL_DSP_STEREO, dsp_stereo); 
+              ioctl(audio, SNDCTL_DSP_STEREO, &dsp_stereo); 
             }
           }
           else {		/* there was extended block */
@@ -397,7 +464,7 @@ void vplay (int fd, int ofs, char *name)
             }
             was_extended = 0;
           }
-          set_dsp_speed (dsp_speed);
+          set_dsp_speed (&dsp_speed);
           break;
         case 2:			/* nothing to do, pure data */
           d_printf ((stderr, "Voice continuation\n"));
@@ -407,9 +474,9 @@ void vplay (int fd, int ofs, char *name)
           dsp_speed = (int)(*data); COUNT(1);
           dsp_speed = 1000000 / (256 - dsp_speed);
           sync_dsp();
-          set_dsp_speed (dsp_speed);
+          set_dsp_speed (&dsp_speed);
           silence = ( ((u_long)*sp) * 1000) / dsp_speed; 
-          d_printf ((stderr, "Silence for %d ms\n", silence));
+          d_printf ((stderr, "Silence for %ld ms\n", silence));
           write_zeros (*sp);
           break;
         case 4:			/* a marker for syncronisation, no effect */
@@ -524,8 +591,7 @@ void start_voc(int fd, u_long cnt)
   Voice_data vd;
   Ext_Block  eb;
  
-  strncpy((char *)vh.magic,MAGIC_STRING,20);
-  vh.magic[19] = 0x1A;
+  strcpy((char *)vh.magic,MAGIC_STRING);
   vh.headerlen = sizeof(VocHeader);
   vh.version = ACTUAL_VERSION;
   vh.coded_ver = 0x1233 - ACTUAL_VERSION;
@@ -590,6 +656,39 @@ void end_wave_raw(int fd)
     close (fd);
 }
 
+void start_snd (int fd, u_long count)
+{
+  SndHeader snd;
+  char *sndinfo = "Recorded by vrec\000";
+
+  snd.magic = SND_MAGIC;
+  snd.dataLocation = sizeof(SndHeader) + strlen(sndinfo);
+  snd.dataSize = count;
+  switch(samplesize){
+  case 8:
+    snd.dataFormat = SND_FORMAT_LINEAR_8;
+    break;
+  case 16:
+    snd.dataFormat = SND_FORMAT_LINEAR_16;
+    break;
+  default:
+    fprintf(stderr,"%d bit: unsupported sample size for NeXt sound file!\n", 
+	    samplesize);
+    exit (-1);
+  }
+  snd.samplingRate = dsp_speed;
+  snd.channelCount = dsp_stereo ? 2 : 1;
+
+  write(fd, &snd, sizeof(SndHeader));
+  write(fd, sndinfo, strlen(sndinfo));
+}
+
+void end_snd (int fd)
+{
+  if (fd != 1)
+    close (fd);
+}
+
 /* playing/recording raw data, this proc handels WAVE files and
    recording .VOCs (as one block) */ 
 void recplay (int fd, int loaded, u_long count, int rtype, char *name)
@@ -601,23 +700,14 @@ void recplay (int fd, int loaded, u_long count, int rtype, char *name)
   int tmps;
 
   sync_dsp();
-  if (!quiet_mode) {
-    fprintf (stderr, "%s %s : ", 
-             (direction == PLAY) ? "Playing" : "Recording",
-             fmt_rec_table[rtype].what);
-    if (samplesize != 8)
-      fprintf(stderr, "%d bit, ", samplesize);
-    fprintf (stderr, "Speed %d Hz ", dsp_speed);
-    fprintf (stderr, "%s ...\n", dsp_stereo ? "Stereo" : "Mono");
-  }
   tmps = samplesize;
-  IOCTL(audio, SNDCTL_DSP_SAMPLESIZE, tmps);
+  ioctl(audio, SNDCTL_DSP_SAMPLESIZE, &tmps);
   if (tmps != samplesize) {
     fprintf (stderr, "%s: unable to set %d bit sample size", 
              command, samplesize);
     if (samplesize == 16) {
       samplesize = 8;
-      IOCTL(audio, SNDCTL_DSP_SAMPLESIZE, samplesize);
+      ioctl(audio, SNDCTL_DSP_SAMPLESIZE, &samplesize);
       if (samplesize != 8) {
         fprintf(stderr, "%s: unable to set 8 bit sample size!\n", command);
         exit (-1);
@@ -646,7 +736,17 @@ void recplay (int fd, int loaded, u_long count, int rtype, char *name)
       exit (-1);
     }
   }
-  set_dsp_speed (dsp_speed);
+  set_dsp_speed (&dsp_speed);
+  if (!quiet_mode) {
+    fprintf (stderr, "%s %s : ", 
+             (direction == PLAY) ? "Playing" : "Recording",
+             fmt_rec_table[rtype].what);
+    if (samplesize != 8)
+      fprintf(stderr, "%d bit, ", samplesize);
+    fprintf (stderr, "Speed %d Hz ", dsp_speed);
+    fprintf (stderr, "%d bits ", samplesize);
+    fprintf (stderr, "%s ...\n", dsp_stereo ? "Stereo" : "Mono");
+  }
  
   if (direction == PLAY) {
     while (count) {
@@ -658,13 +758,22 @@ void recplay (int fd, int loaded, u_long count, int rtype, char *name)
       if ((l = read (fd, (char *)audiobuf + loaded, c - loaded)) > 0) {
         l += loaded; loaded = 0;	/* correct the count; ugly but ... */
         real_l = (one_chn || to_8) ? one_channel(audiobuf, l, one_chn, to_8) : l;
+
+	/* change byte order if necessary */
+	if(convert && (samplesize==16)){
+	  long i;
+
+	  for(i=0; i< real_l; i+=2)
+	    *( (short*) (audiobuf+i) ) = htons( *( (short*) (audiobuf+i)));
+	}
+
         if (write (audio, (char *)audiobuf, real_l) != real_l) {
 	  perror (AUDIO);
 	  exit (-1);
 	}
         count -= l;
       }
-      else {
+      else{
 	if (l == -1) { 
 	  perror (name);
           exit (-1);
@@ -702,9 +811,9 @@ void recplay (int fd, int loaded, u_long count, int rtype, char *name)
 void record_play(char *name)
 {
   int fd, ofs;
-
+  
   if (direction == PLAY) {
- 
+    
     if (!name) {
       fd = 0;
       name = "stdin";
@@ -713,21 +822,33 @@ void record_play(char *name)
       perror (name);
       exit (-1);
     }
-    /* read the file header */
-    read (fd, (char *)audiobuf, sizeof(VocHeader));
-    if ( (ofs = test_vocfile (audiobuf) ) >= 0) 
-      vplay (fd, ofs, name);
+    /* Read the smallest header first, then the
+       missing bytes for the next, etc. */
+    
+    /* read SND-header */
+    read (fd, (char *)audiobuf, sizeof(SndHeader) );
+    if ( test_sndfile(audiobuf, fd) >= 0 )
+      recplay(fd, 0, count, SND_FMT, name);
+    
     else {
-      /* read bytes for WAVE-header */
-      read (fd,  (char *)audiobuf + sizeof(VocHeader), 
-            sizeof(WaveHeader) - sizeof(VocHeader) );
-      if (test_wavefile (audiobuf) >= 0)
-        recplay (fd, 0, count, WAVE_FMT, name);
-      else {
-        /* should be raw data */
-        init_raw_data();
-        count = calc_count();
-        recplay (fd, sizeof(WaveHeader), count, RAW_DATA, name);
+      /* read VOC-Header */
+      read (fd, (char *)audiobuf + sizeof(SndHeader), 
+	    sizeof(VocHeader) - sizeof(SndHeader) );
+      if ( (ofs = test_vocfile (audiobuf) ) >= 0) 
+	vplay (fd, ofs, name);
+      
+      else{
+	/* read bytes for WAVE-header */
+	read (fd,  (char *)audiobuf + sizeof(VocHeader), 
+	      sizeof(WaveHeader) - sizeof(VocHeader) );
+	if (test_wavefile (audiobuf) >= 0)
+	  recplay (fd, 0, count, WAVE_FMT, name);
+	else {
+	  /* should be raw data */
+	  init_raw_data();
+	  count = calc_count();
+	  recplay (fd, sizeof(WaveHeader), count, RAW_DATA, name);
+	}
       }
     }
     if (fd != 0)
@@ -739,7 +860,7 @@ void record_play(char *name)
       name = "stdout";
     }
     else {
-      if ((fd = open (name, O_WRONLY | O_CREAT, 0666)) == -1) {
+      if ((fd = open (name, O_WRONLY | O_CREAT | O_TRUNC, 0666)) == -1) {
         perror (name);
         exit (-1);
       }
